/*
   CODES.C - display input characters

   Copyright (C) 2006  Martin C Gregorie
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
   Contact Martin Gregorie at martin@gregorie.org
*/

#define VERSION   "1.1"
#define COPYRIGHT "Martin Gregorie, 2006"

#include <termios.h>
#include <signal.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

extern int errno;

int main(int argc, char *argv[])
{
	char	c;
	char	cntrl_string[10];
	int	i;
	int     reply = 0;

	sigset_t sig, savesig;
	struct termios modes, savemodes;

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-?") == 0
		    || strcmp(argv[i], "-h") == 0
		    || strcmp(argv[i], "--help") == 0)
		{
		        printf("codes %s. (c) %s\n\n", VERSION, COPYRIGHT);
			printf("Syntax: codes -?\n");
			printf("Function: Display input characters\n");
			printf("          as char, numeric and hex\n");
			printf("          ^D terminates the program\n");
			printf("Options:  none\n");
			exit(0);
		}
		else
		{
			fprintf(stderr, 
                        	"codes: %s\n",
                        	strerror(errno));
			exit(1);
		}
	}


	/*
		Block keyboard signals
	*/
	sigaddset(&sig, SIGINT);
	sigaddset(&sig, SIGQUIT);
	sigaddset(&sig, SIGSTOP);
	sigprocmask(SIG_BLOCK, &sig, &savesig);

	/*
		Get current terminal attributes
	*/
	if (tcgetattr(0, &modes) < 0)
	{
		fprintf(stderr,
			"codes: tcgetattr() returned %d - %s\n",
			errno,
                       	strerror(errno));
		exit(1);
	}

	/*
		Save a copy of them to restore later.
		Then change the attributes to set
		character at a time input and turn off echo.
	*/
	savemodes = modes;
	modes.c_cc[VMIN] = 1;
	modes.c_cc[VTIME] = 0;
	modes.c_lflag &= ~ICANON;
	modes.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHOKE);

	/*
		Activate the changes
	*/
	if (tcsetattr(0, TCSAFLUSH, &modes) < 0)
	{
		fprintf(stderr,
			"codes: tcsetattr() returned %d - %s\n",
			errno,
                       	strerror(errno));
		exit(1);
	}

	/*
		Now handle input characters,
		outputting each as ASCII, decimal and hex
	*/
	printf("Input ctrl/D to quit\n\n");
	while ((reply = read(0, &c, 1)) >= 0)
	{
		/*
			Since non-cannonical mode disables EOF
			we need to handle it ourselves.
		*/
		if (c == savemodes.c_cc[VEOF])
		{
			break;
		}

		/*
			Show what we just read in some detail
		*/
		if (c == 0x07)
		{
			strcpy(cntrl_string, "(Bell)");
		}
		else
		if (c == 0x08)
		{
			strcpy(cntrl_string, "(Backspace)");
		}
		else
		if (c == 0x09)
		{
			strcpy(cntrl_string, "(TAB)");
		}
		else
		if (c == 0x0a)
		{
			strcpy(cntrl_string, "(LF)");
		}
		else
		if (c == 0x0d)
		{
			strcpy(cntrl_string, "(CR)");
		}
		else
		if (c == 0x1b)
		{
			strcpy(cntrl_string, "(Esc)");
		}
		else
		if (c == 0x7f)
		{
			strcpy(cntrl_string, "(DEL)");
		}
		else
		if (iscntrl(c))
		{
			sprintf(cntrl_string, "(ctrl-%c)", 'a' + c - 1);
		}
		else
		{
			strcpy(cntrl_string, "");
		}

		printf("%c = %d  0x%02x  %s\n", 
	       	       (isascii(c) && isprint(c) ? c : '.'),
                        c,
                        c,
			cntrl_string);

	}

	if (reply >= 0)
	{
		printf("codes: run ended\n");
	}
	else
	{
		fprintf(stderr, 
                        "codes: %s\n",
                        strerror(errno));
	}

	/*
		Clean up time.
		Restore the original terrminal attributes
	*/
	if (tcsetattr(0, TCSAFLUSH, &savemodes) < 0)
	{
		fprintf(stderr,
			"codes: tcsetattr() returned %d - %s\n",
			errno,
                       	strerror(errno));
		exit(1);
	}

	/*
		Restore the original signal mask
	*/
	sigprocmask(SIG_SETMASK, &savesig, (sigset_t *)0);

	exit(errno);
}

